''**********************************************************************************
''*  PAL TV Video Demo                                                                *
''*  Original Author: Rayman @    http://www.rayslogic.com/propeller/propeller.htm *
''*  Author Dr Bulb                                                                *
''*  Uses TV.spin by Chip Gracey                                                   *                                       
''**********************************************************************************

' I wanted basic output for my prop experiments to a PAL TV and wanted it quickly.
' Having looked at Raymans PSM_TvBmpViewer I decided I could turn it into a simple
' front end program to use TV.spin for data output. So I carefully pulled all the
' Bitmap code out of the original file and changed TV Settings for my needs ie PAL TV.
' Just change TV Settings variable values for your needs ie NTSC etc.
' Some nice methods for outputting characters and strings were provided in Raymans file
' which make it very nice for quick and easy data output to a TV, so thanks to him.
' Credit to Chip Gracey for the Propeller chip & TV.spin, 3 resistors a phono coax lead
' and presto data output to a TV, man thats just great!
' The output is on the port pins P12, P13 and P14. Thats physical pins 17,18 and 19.
' Hope this is useful to other people like me new to the Propeller Chip. 


CON

  _clkmode = xtal1 + pll16x
  _xinfreq =4_000_000
  _STACK=40      'going to reserve a little stack space, otherwise things can go very wrong!
  x_tiles = 16   'Changing these too much will screw the tv picture up see warning in tvparams section below
  y_tiles = 16   'This was 12 I changed it to 16 and works ok, but 24 caused a screw up, I guess overscan or something.


OBJ

  tv    : "TV"
    


VAR

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64] 

  long  row,col,flag,screensize,rows,cols,color



PUB start | i, pBmp
  'Start up the TV driver and show some text
  longmove(@tv_status, @tvparams, 14)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)   

  rows:=y_tiles
  cols:=x_tiles
  screensize:=rows*cols
  'setcolors(@palette)       'this code commented out to save space, adding the following 2 lines instead...
  colors[0]:=$070A070A       ' these two lines must set Text to white on blue background ?
  colors[1]:=$07070a0a
  out(0)                     ' Clear the screen
  str(string("Propeller Chip"))  'Output a string
  out($0D)                       'Return
  str(string("PAL TV Video"))
  out($0D)
  str(string("on AV1 input"))
  out($0D)
  str(string("xtal1 = 4mhz"))
  out($0D)
  str(string("pll16x"))
  out($0D)
  str(string("= 64mhz Clock"))
  out($0D)
  str(string("Stable H&V Sync"))

  
    
PUB out(c) | i, k

'' Output a character
''
''     $00 = clear screen
''     $01 = home
''     $08 = backspace
''     $09 = tab (8 spaces per)
''     $0A = set X position (X follows)
''     $0B = set Y position (Y follows)
''     $0C = set color (color follows)
''     $0D = return
''  others = printable characters

  case flag
    $00: case c
           $00: wordfill(@screen, $220, screensize)
                col := row := 0
           $01: col := row := 0
           $08: if col
                  col--
           $09: repeat
                  print(" ")
                while col & 7
           $0A..$0C: flag := c
                     return
           $0D: newline
           other: print(c)
    $0A: col := c // cols
    $0B: row := c // rows
    $0C: color := c & 7
  flag := 0

  
PRI print(c)|k,i
      k := color << 1 + c & 1
      i := k << 10 + $200 + c & $FE
      screen[row * cols + col] := i
      screen[(row + 1) * cols + col] := i | 1
      if ++col == cols
        newline


PRI newline : i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
      wordmove(@screen[i*cols], @screen[(i+2)*cols], cols)
    'clear new line
    wordfill(@screen[(rows-2)*cols], $220, cols<<1)


PUB str(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    out(byte[stringptr++])


PUB dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    out("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      out(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      out("0")
    i /= 10

 
PUB hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))



DAT       'TV Settings
          ' Change the values of these variables for your own use, have a look at the the Parameter descriptions
          ' in TV.spin to understand what these variables do and the value ranges required, pretty straightforward.
          ' Just be careful not to get to far away from the required Horizontal linescan frequency, you could blow
          ' the line stage in your tv. If you get screwed up picture and a nasty noise from inside the tv that is probably
          ' the line output transformer protesting at the frequency the line stage is running at, so turn off tv right away
          ' and check/change your code. As is TV.spin works fine on PAL with the variables set as below, cannot speak for
          ' NTSC TV as I am in UK. Am sure the guys on the Propeller Forum from the states can confer that NTSC works just fine.
          ' All I did was change the MODE variable form %0000 NTSC to %0001 PAL and changed nothing else.  
          
tvparams                long    0               'status
                        long    1               'enable
                        'long    %010_0101       'pins for Propeller System Module
                        long    %001_0101       'pins for Demo Board
                        long    %0001           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog


DAT  'text palette
                        '       fore   back
                        '       color  color
palette                 byte    $07,   $0A    '0    white / dark blue
                        byte    $07,   $BB    '1    white / red
                        byte    $9E,   $9B    '2   yellow / brown
                        byte    $04,   $07    '3     grey / white
                        byte    $3D,   $3B    '4     cyan / dark cyan
                        byte    $6B,   $6E    '5    green / gray-green
                        byte    $BB,   $CE    '6      red / pink
                        byte    $3C,   $0A    '7     cyan / blue


DAT    'Palette

TV_Palette long
byte    $02,$03,$04,$05,$06,$07
byte    $08,$0A,$0B,$0c,$0d,$0e
byte    $18,$1A,$1B,$1C,$1D,$1E
byte    $28,$2A,$2B,$2C,$2D,$2E
byte    $38,$3A,$3B,$3C,$3D,$3E
byte    $48,$4A,$4B,$4C,$4D,$4E
byte    $58,$5A,$5B,$5C,$5D,$5E
byte    $68,$6A,$6B,$6C,$6D,$6E
byte    $78,$7A,$7B,$7C,$7D,$7E
byte    $88,$8A,$8B,$8C,$8D,$8E
byte    $98,$9A,$9B,$9C,$9D,$9E
byte    $A8,$AA,$AB,$AC,$AD,$AE
byte    $B8,$BA,$BB,$BC,$BD,$BE
byte    $C8,$CA,$CB,$CC,$CD,$CE
byte    $D8,$DA,$DB,$DC,$DD,$DE
byte    $E8,$EA,$EB,$EC,$ED,$EE
byte    $F8,$FA,$FB,$FC,$FD,$FE